use strict;
use warnings FATAL => 'all';
use utf8;

package Test::Chai::Test::Throw::ReferenceError;
use parent 'Exception::Tiny';

package Test::Chai::Test::Throw::TypeError;
use parent 'Exception::Tiny';

1;
